@@ -1,9 +1,11 @@
 use std::{assert_eq, iter::once};
 use delaunator::{EMPTY, Triangulation, EPSILON};
 use utils::triangle_of_edge;
-use super::{ClipBehavior, Point, bounding_box::{self, *}, iterator::EdgesAroundSiteIterator, utils::{self, site_of_incoming}};
-use approx::abs_diff_eq;
+use crate::utils::abs_diff_eq;
 
+use super::{ClipBehavior, Point, bounding_box::{self, *}, iterator::EdgesAroundSiteIterator, utils::{self, site_of_incoming}};
+
+#[derive(Debug)]
 pub struct CellBuilder {
     vertices: Vec<Point>,
     bounding_box: BoundingBox,
@@ -56,6 +58,33 @@ impl CellBuilder {
             }
         },"The first vertex of each voronoi cell on the hull must always be the circumcenter of the triangle associated to the leftmost incoming half-edge.");
 
+        debug_assert!({
+            let invalid_hull_sites: Vec<usize> = hull.iter().copied().filter(|&hull_site| {
+                triangulation.halfedges[site_to_incoming_leftmost_halfedge[hull_site]] != EMPTY
+            }).collect();
+
+            if !invalid_hull_sites.is_empty() {
+                println!("The following hull sites have incorrect leftmost halfedge cached: {:?}", invalid_hull_sites);
+                false
+            } else {
+                true
+            }
+        },"The left most edge for hull sites must not have a half-edge.");
+
+        debug_assert!({
+            let invalid_hull_sites: Vec<usize> = site_to_incoming_leftmost_halfedge.iter().enumerate()
+                .filter(|(site, &edge)| triangulation.halfedges[edge] == EMPTY && !hull.contains(site))
+                .map(|(site, _)| site)
+                .collect();
+
+            if !invalid_hull_sites.is_empty() {
+                println!("The following sites are not on the hull list but they are supposed to because they have an edge without an associated half-edge: {:?}", invalid_hull_sites);
+                false
+            } else {
+                true
+            }
+        },"The all sites whose left most edge has no associated half-edge must be on the hull.");
+
         self.extend_and_close_hull(sites, &hull, &mut cells);
 
         // clip cells
@@ -77,13 +106,13 @@ impl CellBuilder {
         // the vertex is the circumcenter of the triangle of edge a->b
         // the vertex is on the a->b bisector line, thus we can take midpoint of a->b and vertex for the orthogonal projection
         let edge_midpoint = Point { x: (site_a.x + site_b.x) / 2.0, y: (site_a.y + site_b.y) / 2.0 };
-        
+
         // clockwise rotation 90 degree from edge direction
         let mut orthogonal = Point { x: site_b.y - site_a.y, y: site_a.x - site_b.x };
         // normalizing the orthogonal vector
         let ortho_length = (orthogonal.x * orthogonal.x + orthogonal.y * orthogonal.y).sqrt();
-        orthogonal.x = orthogonal.x * (1. / ortho_length);
-        orthogonal.y = orthogonal.y * (1. / ortho_length);
+        orthogonal.x *= 1. / ortho_length;
+        orthogonal.y *= 1. / ortho_length;
 
         let projected = Point { x: edge_midpoint.x + (scale * orthogonal.x), y: edge_midpoint.y + (scale * orthogonal.y) };
         let index = self.vertices.len();
@@ -158,7 +187,9 @@ impl CellBuilder {
                 previous = Some(a);
 
                 // remove vertex if it is empty
-                let vertex = if a == EMPTY {
+                
+
+                if a == EMPTY {
                     if prev == Some(EMPTY) {
                         // two EMPTY vertices removed in a row means an edge was removed
                         last_edge_removed = true;
@@ -189,9 +220,7 @@ impl CellBuilder {
 
                     vertex_count += 1;
                     Some(a)
-                };
-
-                vertex
+                }
             })
             .collect::<Vec<usize>>();
 
@@ -227,7 +256,14 @@ impl CellBuilder {
             };
 
             // this many vertices were added between A and B, this means values were shifted right
-            open_edge_index_adjustment += self.link_vertices_around_box_edge(cell, a, b);
+            let new_vertices = self.link_vertices_around_box_edge(cell, a, b);
+
+            // only add to the adjustment if this is not the last-first edge
+            // because any new vertices will be added to the end of the array
+            // thus they do not shift the position of the other vertices
+            if open_edges[i] != 0 {
+                open_edge_index_adjustment += new_vertices;
+            }
         }
 
         // if after closing, we are left with a line, this cell should be removed
@@ -272,7 +308,7 @@ impl CellBuilder {
                             // a -> box -> b; edge crosses box (case 1,2,3), we clip this ray twice
                             new_b = v_index + 1;
 
-                            if abs_diff_eq!(clip_a.x, clip_b.x, epsilon = EPSILON) && abs_diff_eq!(clip_a.y, clip_b.y, epsilon = EPSILON) {
+                            if abs_diff_eq(clip_a.x, clip_b.x, EPSILON) && abs_diff_eq(clip_a.y, clip_b.y, EPSILON) {
                             // Sadly, the nearly_equals-function is private in delaunator
                             //if clip_a.nearly_equals(clip_b) {
                                 // case 3 - a and b outside box, intersection at the corner
@@ -319,9 +355,17 @@ impl CellBuilder {
 
             // track new index for b
             new_b = self.vertices.len();
+            debug_assert!(clip_b.is_some(), "When clipping a [inside] ({:?}) -> b ({:?}) [outside], the projected vertex b was expected to be on the bounding box's edge ({:?}). However no projection was returned.", pa, pb, self.bounding_box);
             self.vertices.push(clip_b.expect("Vertex 'b' is outside the bounding box. An intersection should have been returned."));
         } // else neither is outside, not need for clipping
 
+        fn assert_on_box_edge(builder: &CellBuilder, i: usize) -> bool {
+            let (top, left) = builder.bounding_box.which_edge(&builder.vertices[i]);
+            left != BoundingBoxLeftRightEdge::None || top != BoundingBoxTopBottomEdge::None
+        }
+        debug_assert!(new_a == a || new_a == EMPTY || assert_on_box_edge(self, new_a), "Edge {} ({:#?}) -> {} ({:#?}) was clipped but endpoint {} ({:#?}) is not on box's edge {:#?}.", a, self.vertices[a], b, self.vertices[b], new_a, self.vertices[new_a], self.bounding_box.which_edge(&self.vertices[new_a]));
+        debug_assert!(new_b == b || new_b == EMPTY || assert_on_box_edge(self, new_b), "Edge {} ({:#?}) -> {} ({:#?}) was clipped but endpoint {} ({:#?}) is not on box's edge {:#?}.", a, self.vertices[a], b, self.vertices[b], new_b, self.vertices[new_b], self.bounding_box.which_edge(&self.vertices[new_b]));
+
         (new_a, new_b)
     }
 
@@ -337,8 +381,8 @@ impl CellBuilder {
         let (b_bt, b_lr) = self.bounding_box.which_edge(pb);
 
         // It is expected that points ARE on the box edge
-        debug_assert!(!(a_bt == BoundingBoxTopBottomEdge::None && a_lr == BoundingBoxLeftRightEdge::None), "Point a ({}) was not located on the box's edge", a);
-        debug_assert!(!(b_bt == BoundingBoxTopBottomEdge::None && b_lr == BoundingBoxLeftRightEdge::None), "Point b ({}) was not located on the box's edge", b);
+        assert!(!(a_bt == BoundingBoxTopBottomEdge::None && a_lr == BoundingBoxLeftRightEdge::None), "Point a ({}) [{:?}] was not located on any of the box's edge", a, pa);
+        assert!(!(b_bt == BoundingBoxTopBottomEdge::None && b_lr == BoundingBoxLeftRightEdge::None), "Point b ({}) [{:?}] was not located on any of the box's edge", b, pb);
 
         // short-circuit for case where a and b are on the same edge, and b is ahead of a, such case there is nothing to do
         if (match (a_bt, b_bt) {
@@ -467,7 +511,7 @@ fn build_cells(sites: &Vec<Point>, triangulation: &Triangulation, site_to_incomi
         // triangle[edge] is the site 'edge' originates from, but EdgesAroundPointIterator
         // iterate over edges around the site 'edge' POINTS TO, thus to get that site
         // we need to take the next half-edge
-        let site = site_of_incoming(&triangulation, edge);
+        let site = site_of_incoming(triangulation, edge);
 
         // if we have already created the cell for this site, move on
         if !seen_sites[site] {
@@ -483,8 +527,8 @@ fn build_cells(sites: &Vec<Point>, triangulation: &Triangulation, site_to_incomi
 
             let cell = &mut cells[site];
             cell.extend(
-                EdgesAroundSiteIterator::new(&triangulation, leftmost_edge)
-                        .map(|e| utils::triangle_of_edge(e))
+                EdgesAroundSiteIterator::new(triangulation, leftmost_edge)
+                        .map(utils::triangle_of_edge)
             );
         }
     }
@@ -502,7 +546,7 @@ mod test {
 
     fn assert_same_elements(actual: &Vec<usize>, expected: &Vec<usize>, message: &str) {
         assert_eq!(actual.len(), expected.len(), "Vectors differ in length. Actual: {:?}. Expected: {:?}. {}", actual, expected, message);
-        assert_eq!(0, actual.iter().copied().zip(expected.iter().copied()).filter(|(a,b)| a != b).collect::<Vec<(usize, usize)>>().len(), "Vectors have differing elements. Actual: {:?}. Expected: {:?}. {}", actual, expected, message);
+        assert_eq!(0, actual.iter().copied().zip(expected.iter().copied()).filter(|(a,b)| a != b).count(), "Vectors have differing elements. Actual: {:?}. Expected: {:?}. {}", actual, expected, message);
     }
 
     fn assert_cell_vertex_without_bounds(builder: &CellBuilder, cell: &Vec<usize>, message: &str, expected_vertices: Vec<Point>) {
@@ -525,7 +569,7 @@ mod test {
 
         // are all points within the bounding box?
         let points_outside: Vec<(usize, &Point)> = points.iter().enumerate().filter(|(_, p)| {
-            !builder.bounding_box.is_inside(&p)
+            !builder.bounding_box.is_inside(p)
         }).collect();
         assert_eq!(0, points_outside.len(), "These points are outside bounding box: {:?}. {}", points_outside, message);
 
@@ -556,7 +600,7 @@ mod test {
             Point { x: 1.0, y: 2.0 },
         ]);
         let original_cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut cell = original_cell.clone();
+        let mut cell = original_cell;
         builder.calculate_corners();
         builder.link_vertices_around_box_edge(&mut cell, 0, 1);
         assert_same_elements(&cell, &vec![0, builder.top_left_corner_index, builder.bottom_left_corner_index, builder.bottom_right_corner_index, builder.top_right_corner_index, 1], "Full circuit expected because b is before a");
@@ -569,7 +613,7 @@ mod test {
             Point { x: 2.0, y: 2.0 },
         ]);
         let original_cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut cell = original_cell.clone();
+        let mut cell = original_cell;
         builder.calculate_corners();
         builder.link_vertices_around_box_edge(&mut cell, 0, 1);
         assert_same_elements(&cell, &vec![0, builder.top_left_corner_index, builder.bottom_left_corner_index, builder.bottom_right_corner_index, 1], "Corners not expected to be duplicated");
@@ -582,7 +626,7 @@ mod test {
             Point { x: 2.0, y: -2.0 },
         ]);
         let original_cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut cell = original_cell.clone();
+        let mut cell = original_cell;
         builder.calculate_corners();
         builder.link_vertices_around_box_edge(&mut cell, 0, 1);
         assert_same_elements(&cell, &vec![0, builder.bottom_left_corner_index, 1], "Corners not expected to be duplicated");
@@ -595,7 +639,7 @@ mod test {
             Point { x: -2.0, y: 1.5 },
         ]);
         let original_cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut cell = original_cell.clone();
+        let mut cell = original_cell;
         builder.calculate_corners();
         builder.link_vertices_around_box_edge(&mut cell, 0, 1);
         assert_same_elements(&cell, &vec![0, builder.top_right_corner_index, builder.top_left_corner_index, 1], "Corners not expected to be duplicated");
@@ -608,7 +652,7 @@ mod test {
             Point { x: 2.0, y: 1.5 },
         ]);
         let original_cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut cell = original_cell.clone();
+        let mut cell = original_cell;
         builder.calculate_corners();
         builder.link_vertices_around_box_edge(&mut cell, 0, 1);
         assert_same_elements(&cell, &vec![0, builder.bottom_left_corner_index, builder.bottom_right_corner_index, 1], "Corners not expected to be duplicated");
@@ -622,7 +666,7 @@ mod test {
             Point { x: -1.5, y: 2.0 },
         ]);
         let original_cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut cell = original_cell.clone();
+        let mut cell = original_cell;
         builder.calculate_corners();
         builder.link_vertices_around_box_edge(&mut cell, 0, 1);
         assert_same_elements(&cell, &vec![0, builder.bottom_right_corner_index, builder.top_right_corner_index, 1], "Corners not expected to be duplicated");
@@ -673,7 +717,7 @@ mod test {
             Point { x: 0.0, y: 1.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.clip_and_close_cell(&mut clipped_cell);
         assert_same_elements(&clipped_cell, &vec![0, 1, 2], "No clipping expected");
         assert_cell_consistency(&clipped_cell, &builder, "Cell consistency check");
@@ -687,7 +731,7 @@ mod test {
             Point { x: 1.0, y: 0.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.clip_and_close_cell(&mut clipped_cell);
         assert_same_elements(&clipped_cell, &vec![0, 3, 4, 2], "Clipped cell incorrect indices.");
         assert_eq!(Point { x: 0.0, y: -2.0 }, builder.vertices[3], "Point should have been added for clipped edge.");
@@ -703,7 +747,7 @@ mod test {
             Point { x: 0.0, y: -3.0 }, // outside
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.clip_and_close_cell(&mut clipped_cell);
         assert_same_elements(&clipped_cell, &vec![0, 1, 3, 4], "Clipped cell incorrect indices.");
         assert_eq!(Point { x: 0.0, y: -2.0 }, builder.vertices[3], "Point should have been added for clipped edge.");
@@ -719,7 +763,7 @@ mod test {
             Point { x: 20.0, y: 0.0 }, // comes back through the right
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         builder.clip_and_close_cell(&mut clipped_cell);
         // builder adds 4 vertices all the time, so next index is 7
@@ -737,7 +781,7 @@ mod test {
             Point { x: 3.0, y: 0.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         builder.clip_and_close_cell(&mut clipped_cell);
         // builder adds 4 vertices all the time, so next index is 7
@@ -758,7 +802,7 @@ mod test {
             Point { x: 0.0, y: 4.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         builder.clip_and_close_cell(&mut clipped_cell);
         assert_same_elements(&clipped_cell, &vec![0, 7, builder.top_right_corner_index , 8], "Clipped cell incorrect indices.");
@@ -776,7 +820,7 @@ mod test {
             Point { x: 0.0, y: 4.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, false, "A single intersection at the coner with all other points outside the cell, should not keep this cell.");
@@ -791,7 +835,7 @@ mod test {
             Point { x: 2.0, y: 4.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         builder.clip_and_close_cell(&mut clipped_cell);
         // FIXME: identify intersection at corners and return corner index instead of creating a new one
@@ -811,7 +855,7 @@ mod test {
             Point { x: 2.0, y: -4.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_same_elements(&clipped_cell, &vec![7, 8], "Clipped cell incorrect indices.");
@@ -830,7 +874,7 @@ mod test {
             Point { x: 2.0, y: -2.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         builder.clip_and_close_cell(&mut clipped_cell);
         assert_same_elements(&clipped_cell, &vec![7, 1, 2, 8], "Clipped cell incorrect indices.");
@@ -855,7 +899,7 @@ mod test {
             Point { x: 15.0, y: 0.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, false, "No intersection with box, all points outside, should not keep the cell.");
@@ -871,7 +915,7 @@ mod test {
             Point { x: 0.0, y: 5.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, true, "Intersection with box.");
@@ -892,7 +936,7 @@ mod test {
             Point { x: 5.0, y: 5.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, true, "Intersection with box.");
@@ -918,7 +962,7 @@ mod test {
             Point { x: 0.0, y: 5.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, true, "Intersection with box.");
@@ -949,7 +993,7 @@ mod test {
             Point { x: 0.0, y: 5.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, true, "Intersection with box.");
@@ -980,7 +1024,7 @@ mod test {
             Point { x: 5.0, y: 5.0 },
         ]);
         let cell: Vec<usize> = (0..builder.vertices.len()).collect();
-        let mut clipped_cell = cell.clone();
+        let mut clipped_cell = cell;
         builder.calculate_corners();
         let keep_cell = builder.clip_and_close_cell(&mut clipped_cell);
         assert_eq!(keep_cell, true, "Intersection with box.");
